@@ -1384,13 +1384,12 @@ function (x, rw = 1:x@dimension[1], cl = 1:x@dimension[2], value)
 	if(!all(abs(rw[,1])%in%1:nrow)||!all(abs(rw[,2])%in%1:ncol))
 		stop("Subscripts out of bound")
 	s <- match(paste(rw[,1],rw[,2]),paste(x@ia,x@ja))
-#	ra <- c(x@ra[-s],value@ra[!is.na(s)])
-#        ja <- as.integer(c(x@ja[-s],rw[,2][!is.na(s)]))
-#        ia <- as.integer(c(x@ia[-s],rw[,1][!is.na(s)]))
-	value <- as.matrix.csr(value)
-	ra <- c(x@ra[-s],value@ra)
-        ja <- as.integer(c(x@ja[-s],rw[,2]))
-        ia <- as.integer(c(x@ia[-s],rw[,1]))
+	if(length(value)==1) value <- rep(value,nrow(rw))
+	if(length(value)!=nrow(rw))
+		stop("assignment value has incompatible length")
+	ra <- c(x@ra[-s[!is.na(s)]],value)
+        ja <- as.integer(c(x@ja[-s[!is.na(s)]],rw[,2]))
+        ia <- as.integer(c(x@ia[-s[!is.na(s)]],rw[,1]))
         dim <- x@dimension
         x <- new("matrix.coo",ra=ra,ja=ja,ia=ia,dimension=dim)
 	}
